/****************************************************************************

amzi.h - standard prolog header for adding predicates in "C"


Copyright (c) 1992-96 by Amzi! inc.  All Rights Reserved.

****************************************************************************/

/* Environment dependent defines */

#ifdef __cplusplus
extern "C"
{
#endif

/* Watcom */
/* ------ */

#ifdef __WATCOMC__

#define IBMPC
#define WC

#ifdef __386__
#define P32
#else
#define P16
#endif

#ifdef __WINDOWS__
#define MSWIN
#include <windows.h>
#else
#define DOS
#endif

#endif

/* Microsoft */
/* --------- */

#ifdef _MSC_VER

#define IBMPC
#define MSC

#ifdef _WIN32
#define P32
#else
#define P16
#endif

#if defined(_WINDOWS) || defined(_WIN32)
#define MSWIN
#include <windows.h>
#else
#define DOS
#endif

#endif /* _MSC_VER */

/* Borland */
/* ------- */

#ifdef __BORLANDC__

#define IBMPC
#define BC

#ifdef __WIN32__
#define P32
#else
#define P16
#endif

#ifdef _Windows
#define MSWIN
#else
#define DOS
#endif

#endif

/* DEC Alpha */
/* --------- */

#ifdef __osf__
#define P64            /* 64-bit flat pointers */
#define UNIX
#endif

#ifdef __bsdi__
#define P32
#define UNIX
#endif

#ifdef __vms
#define P32            /* 32-bit flat pointers */
#define VMS
#endif

/* Needed for error recovery, Prolog errors will do a longjmp to the
   location marked by the setjmp after the call to lsLoad in the main
   calling program.  Not compatible with C++ */
   
#if !defined(__cplusplus)
#include <setjmp.h>
#endif

/* Basic Types & Defines */

#if !(defined(MSWIN)&&defined(WC))
typedef   unsigned char  BYTE;
#endif

typedef   BYTE *      BYTEptr;

typedef  int   RC;          /* intended to be 0 for OK, else err no. */
#define  OK     0

typedef int TF;             /* intended for Prolog returns */
#define TRUE  1
#define FALSE 0

#define  EOS    '\0'

typedef   char *      STRptr;
typedef   STRptr *    STRhnd;

typedef   void *      VOIDptr;
typedef   VOIDptr *   VOIDhnd;

/* Integer types - these must keep the relationship that intC is an
   integer of the same size as CELL, intCH is an integer of half the
   size of a CELL and intCQ is an integer of a quarter of the size of
   a CELL */

#if defined(P64)

typedef long              intC;
typedef unsigned long    uintC;
typedef int              intCH;
typedef unsigned int    uintCH;
typedef short            intCQ;
typedef unsigned short  uintCQ;
typedef short            INT16;
typedef int              INT32;

#elif defined(P32)

typedef long              intC;
typedef unsigned long    uintC;
typedef short            intCH;
typedef unsigned short  uintCH;
typedef char             intCQ;
typedef unsigned char   uintCQ;
typedef short            INT16;
typedef long             INT32;

#elif defined(P16)

typedef long              intC;
typedef unsigned long    uintC;
typedef short            intCH;
typedef unsigned short  uintCH;
typedef char             intCQ;
typedef unsigned char   uintCQ;
typedef short            INT16;
typedef long             INT32;

#endif

typedef intC *        intCptr;
typedef intCH *       intCHptr;
typedef uintC *       uintCptr;

/* Various WAM types */

typedef uintC      CELL;
typedef CELL *     CELLptr;

typedef CELLptr    TERM;
typedef TERM *     TERMptr;

typedef intCH      PATOM;             /* Prolog Atom */
typedef PATOM *    PATOMptr;

typedef uintCH     ARITY;
typedef ARITY *    ARITYptr;

typedef enum
{
   pATOM,
   pINT,
   pSTR,
   pFLOAT,
   pSTRUCT,
   pLIST,
   pTERM,
   pADDR,
   pVAR
} pTYPE;

typedef enum
{
   cATOM,
   cSTR,
   cINT,
   cLONG,
   cSHORT,
   cFLOAT,
   cDOUBLE,
   cADDR,
   cTERM
} cTYPE;

typedef enum
{
   CUR_IN,
   CUR_OUT,
   CUR_ERR,
   USER_IN,
   USER_OUT,
   USER_ERR
} STREAM;

#define STR_BUF 512

typedef uintC   ENGid;
typedef ENGid * ENGidptr;

#define   ABORT_ERR  -2       /* passed back in longjmps to caller */
#define   FATAL_ERR  -1
#define   IPL_ERR    -3
#define   PREV_ERR   -4
#define   STR_TRUNC  -5       /* returned by lsTermToStr() if truncated */

#define   USER_RESET  -6       /* sent by the abort predicate */
#define   USER_EXIT   -7
#define   USER_ABORT  -8


/* External function declarations */
/* ------------------------------ */

#if defined(MSWIN) && defined(P16)
#define LSAPI __pascal
#define EXPFUNC __export __pascal
typedef TF (EXPFUNC *ExtPred)(ENGid);

#elif defined(MSWIN) && defined(P32)
#define LSAPI __stdcall
#define EXPFUNC __stdcall
typedef TF (EXPFUNC *ExtPred)(ENGid);

#elif defined(DOS)
#define LSAPI __pascal
#define EXPFUNC __pascal
typedef TF (EXPFUNC *ExtPred)(ENGid);

#else
#define LSAPI
#define EXPFUNC
typedef TF (*ExtPred)(ENGid);

#endif


typedef struct {
     STRptr     Pname;
     ARITY      Parity;
     ExtPred    Pfunc;
} PRED_INIT;
typedef PRED_INIT * PRED_INITptr;


/* Main entry points to set up Prolog environment */

RC  LSAPI  lsInit(ENGidptr, STRptr);
RC  LSAPI  lsInitLSX(ENGid, VOIDptr);
RC  LSAPI  lsAddLSX(ENGid, STRptr, VOIDptr);
RC  LSAPI  lsAddPred(ENGid, STRptr, ARITY, ExtPred);
RC  LSAPI  lsInitPreds(ENGid, PRED_INITptr);
RC  LSAPI  lsLoad(ENGid, STRptr);
TF  LSAPI  lsMain(ENGid);
RC  LSAPI  lsReset(ENGid);
RC  LSAPI  lsClose(ENGid);

/* function and predicate parameters */

RC    LSAPI  lsGetParm(ENGid, int, cTYPE, VOIDptr);
pTYPE LSAPI  lsGetParmType(ENGid, int);
int   LSAPI  lsStrParmLen(ENGid, int);
TF    LSAPI  lsUnifyParm(ENGid, int, cTYPE, VOIDptr);

/* Calling Prolog from C */

TF LSAPI  lsExec(ENGid, TERMptr);
TF LSAPI  lsExecStr(ENGid, TERMptr, STRptr);
TF LSAPI  lsCall(ENGid, TERMptr);
TF LSAPI  lsCallStr(ENGid, TERMptr, STRptr);
TF LSAPI  lsRedo(ENGid);
RC LSAPI  lsClearCall(ENGid);

/* Asserting and retracting */

RC LSAPI  lsAsserta(ENGid, TERM);
RC LSAPI  lsAssertz(ENGid, TERM);
RC LSAPI  lsRetract(ENGid, TERM);
RC LSAPI  lsAssertaStr(ENGid, STRptr);
RC LSAPI  lsAssertzStr(ENGid, STRptr);
RC LSAPI  lsRetractStr(ENGid, STRptr);


/* String/Term conversion functions */

RC LSAPI  lsTermToStr(ENGid, TERM, STRptr, int);
RC LSAPI  lsTermToStrQ(ENGid, TERM, STRptr, int);
RC LSAPI  lsStrToTerm(ENGid, TERMptr, STRptr);

/* Making Prolog types */

RC LSAPI  lsMakeAtom(ENGid, TERMptr, STRptr);
RC LSAPI  lsMakeStr(ENGid, TERMptr, STRptr);
RC LSAPI  lsMakeInt(ENGid, TERMptr, intC);
RC LSAPI  lsMakeFloat(ENGid, TERMptr, float);
RC LSAPI  lsMakeAddr(ENGid, TERMptr, VOIDptr);

/* Getting C values from Prolog terms */

pTYPE LSAPI  lsGetTermType(ENGid, TERM);
RC    LSAPI  lsGetTerm(ENGid, TERM, cTYPE, VOIDptr);

/* Structure hacking functions */

RC  LSAPI  lsGetFA(ENGid, TERM, STRptr, ARITYptr);
RC  LSAPI  lsMakeFA(ENGid, TERMptr, STRptr, ARITY);
TF  LSAPI  lsUnifyArg(ENGid, TERMptr, int, cTYPE, VOIDptr);
RC  LSAPI  lsGetArg(ENGid, TERM, int, cTYPE, VOIDptr);
pTYPE LSAPI  lsGetArgType(ENGid, TERM, int);
int LSAPI  lsStrArgLen(ENGid, TERM, int);
TF  LSAPI  lsUnify(ENGid, TERM, TERM);

/* List hacking functions */

RC LSAPI  lsMakeList(ENGid, TERMptr);
RC LSAPI  lsPushList(ENGid, TERMptr, TERM);
RC LSAPI  lsPopList(ENGid, TERMptr, cTYPE, VOIDptr);

/* Stream I/O functions */

RC  LSAPI  lsSetStream(ENGid, STREAM, int);
int LSAPI  lsGetStream(ENGid, STREAM);

RC  LSAPI  lsSetInput(ENGid, int (*)(), void (*)());
RC  LSAPI  lsSetOutput(ENGid, void (*)(), void (*)());
RC  LSAPI  lsWinIO(ENGid, int(*)(), int(*)(),
                           int(*)(), int(*)(), int(*)() );

/* Miscellaneous functions */

RC  LSAPI  lsGetVersion(ENGid, STRptr);
RC  LSAPI  lsSetCommandArgs(ENGid, int, char**);
RC  LSAPI  lsSetAllocFree(ENGid, VOIDptr(*)(), void(*)());

/* Error handling */

#if !defined(__cplusplus) && !defined(_WIN32)
#ifdef __alpha
long* LSAPI lsGetJmp(ENGid);
#else
jmp_buf* LSAPI  lsGetJmp(ENGid);
#endif
RC      LSAPI  lsClrJmp(ENGid);
#endif
RC LSAPI  lsErrMsg(ENGid, STRptr);
RC LSAPI  lsErrRaise(ENGid, STRptr);
RC LSAPI  lsErrReadBuf(ENGid, intCptr, STRptr);

#ifdef __cplusplus
}
#endif
